package com.malk.ruisi.schedule;

import com.malk.ruisi.service.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;

/**
 * @EnableScheduling 开启定时任务 [配置参考McScheduleTask]
 */
@Slf4j
@Configuration
@EnableScheduling
@ConditionalOnProperty(name = {"enable.scheduling"})
public class ScheduleTask {

    @Autowired
    private LegalEntityService legalEntityService;
    /*公司_法人主体定时器*/
    //每周一晚上凌晨同步数据
    @Scheduled(cron = "0 0 0 * * 1")
    public void LegalEntityList(){
        log.info("开始同步法人主体到本周");
        try{
            legalEntityService.LegalEntity();
            System.out.println("法人主体同步完成");
        }catch (Exception e){
            //记录错误信息
            e.printStackTrace();
        }

    }

    @Autowired
    private ActivityProjectService activityProjectService;
    /*活动项目信息定时器(新增项目接口在ActivityProjectController里)*/
    @Scheduled(cron = "0 0 1 * * 1")
    public void ActivityProjectList(){
        log.info("开始同步活动项目信息到本周");
        try{
            activityProjectService.Activity();
            System.out.println("活动项目信息同步完成");
        }catch (Exception e){
            //记录错误信息
            e.printStackTrace();
        }
    }

    @Autowired
    private BankNameService bankNameService;
    /*行号定时器*/
    @Scheduled(cron = "0 0 4 * * 1")
    public void BankNameList(){
        log.info("开始同步行号信息到本周");
        System.out.println("行号同步完成");
        try{
            bankNameService.BankName();
        }catch (Exception e){
            //记录错误信息
            e.printStackTrace();
        }
    }

    @Autowired
    private CostBearerService costBearerService;
    /*费用承担主体定时器*/
    @Scheduled(cron = "0 0 3 * * 1")
    public void CostBearerList(){
        log.info("开始同步费用承担主体到本周");
        try{
            costBearerService.CostBearer();
            System.out.println("费用承担主体同步完成");
        }catch (Exception e){
            //记录错误信息
            e.printStackTrace();
        }
    }

    @Autowired
    private CostCenterService costCenterService;
    /*成本中心定时器*/
    @Scheduled(cron = "0 40 3 * * 1")
    public void CostCenterList(){
        log.info("开始同步成本中心定时器");
        try {
            costCenterService.CostCenter();
            System.out.println("成本中心同步完成");
        }catch (Exception e){
            //记录错误信息
            e.printStackTrace();
        }
    }



    @Autowired
    private DeleteYDDataService deleteYDDataService;
    /*批量删除宜搭表单数据*/
//    @Scheduled(cron = "0/1 * * * * ?")
    public void DeleteYDData(){
        log.info("开始批量删除宜搭数据");
        try {
            deleteYDDataService.syncDelete();
            System.out.println("批量删除完成");
        }catch (Exception e){
            //记录错误数据
            e.printStackTrace();
        }
    }


}
