package com.malk.ruisi.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.gson.JsonArray;
import com.malk.ruisi.client.WSContext;
import com.malk.ruisi.service.ExpenseAccrualService;
import com.malk.ruisi.services.EASLogin.EASLoginProxyProxy;
import com.malk.ruisi.services.WSPortalMainFacade.WSPortalMainFacadeSrvProxyProxy;
import com.malk.server.aliwork.YDConf;
import com.malk.server.aliwork.YDParam;
import com.malk.server.common.McR;
import com.malk.server.dingtalk.DDConf;
import com.malk.service.aliwork.YDClient;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.SneakyThrows;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.time.Instant;
import java.time.LocalDate;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;


@Service
public class ExpenseAccrualServiceImpl implements ExpenseAccrualService {
    @Autowired
    private YDClient ydClient;
    @Autowired
    private YDConf ydConf;
    @Autowired
    private DDConf ddConf;

    @SneakyThrows
    @Override
    public McR insetXMC(String instanceId) throws JsonProcessingException {
        if(instanceId !=null){
            System.out.println("实例ID："+instanceId);
            Map data = (Map) ydClient.queryData(YDParam.builder().formInstId(instanceId)
                    .appType(ydConf.getAppType()).systemToken(ydConf.getSystemToken())
                    .userId(ddConf.getOperator()).build(), YDConf.FORM_QUERY.retrieve_id).getFormData();
            System.out.println(data);//打印宜搭表单数据

            //处理传入参数类型
            ObjectMapper objectMapper = new ObjectMapper();
            Head head = new Head();
            //1、head表头传参
            head.setSettlementType("5");//固定：不付款的编码5

            head.setNumber(data.get("serialNumberField_lucde59d").toString());//流程编码

            String s = data.get("serialNumberField_lucde59d").toString();
            String requestId = s.substring(s.length() - 7);
            head.setRequestId(requestId);//Portal做唯一性校验

            long time = (long)data.get("dateField_lucde59g");
            String utc = convertTimestampToDateString(time, "UTC", "yyyy-MM-dd");
            head.setAppDate(utc);//申请日期

            String company = data.get("textField_lweul68w").toString();
            head.setCompany(company);//所属公司编码

            head.setAppDepart(data.get("textField_lwa4ipaf").toString());//申请部门编码
            head.setApplicant(data.get("textField_lwa4ipah").toString());//申请人编码
            head.setStatus("审核");//审核状态

            // 创建并设置entry对象列表
            List<ExpenseAccrualServiceImpl.Entry> entries = new ArrayList<>();
            Entry entry = new Entry();
            //获取子表单中字段信息
            List<Map> jsonString = (List<Map>)data.get("tableField_lucde59l");
            System.out.println("jsonString:"+jsonString);
            int num = 0;
            for (Map map1 : jsonString) {
                //财务摘要:"计提-年月-费用承担部门-费用内容"
                LocalDate currentDate = LocalDate.now();
                DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM");
                String currentYearMonth = currentDate.format(formatter);
                map1.get("textField_lx2omt23").toString();
                map1.get("textField_lucde59m").toString();

                entry.setSummary("计提-"+currentYearMonth+map1.get("textField_lx2omt23").toString()+map1.get("textField_lucde59m").toString());//财务摘要

                num++;//子表单存在多条就+1
                String entryOwnId = requestId.concat(String.valueOf(num));//分录唯一id
                entry.setEntryOwnId(entryOwnId);//entryOwnId

                String title = map1.get("textField_lucde59m").equals(null) ? " " : map1.get("textField_lucde59m").toString();
                entry.setTitle(title);//费用内容

                String s1 = map1.get("associationFormField_lucde59t_id").toString();
                JSONArray jsonArray = JSONArray.parseArray(s1.substring(1, s1.length() - 1).replaceAll("\\\\",""));
                Map map = (Map) jsonArray.get(0);
                String lines = map.get("title").toString();
                entry.setLines(lines);//产品线

                String portalFeeType = map1.get("textField_lwf2vn9y").toString();
                entry.setPortalFeeType(portalFeeType);//费用类型

                String proNum = map1.get("textField_lweqdqnf").toString();
                entry.setProNum(proNum);//活动项目编号

                String costCenter = map1.get("textField_lweqdqng").toString();
                entry.setCostCenter(costCenter);//成本中心编码

                String campus = map1.get("textField_lx2niu81").toString();
                entry.setCampus(campus);//校区部门编码

                String payCompany = map1.get("textField_lwa4ipai").toString();
                entry.setPayCompany(payCompany);//付款主体

                String currency = map1.get("textField_lwerwbwv").toString();
                entry.setCurrency(currency);//币别

                String exchangeRate = new BigDecimal(map1.get("numberField_lucde59v").toString())
                        .setScale(2, BigDecimal.ROUND_HALF_UP).toString();
                entry.setExchangeRate(exchangeRate);//汇率

                String amount = map1.get("numberField_lucde59w").toString();
                entry.setAmount(amount);//原币金额

                String realAmount = map1.get("numberField_lucde59x").toString();
                entry.setRealAmount(realAmount);//折合人民币金额

                String supplierNo = map1.get("textField_lx2niu82").toString();
                entry.setSupplierNo(supplierNo);//供应商编码

                entries.add(entry);

            }
            Data data1 = new Data();
            data1.setHead(head);
            data1.setEntry(entries);

            MyJson json = new MyJson();
            json.setType("CW13");//type
            ArrayList<Data> dataList = new ArrayList<>();
            dataList.add(data1);
            json.setData(dataList);//data
            // 序列化对象到JSON字符串
            String jsonString1 = objectMapper.writeValueAsString(json);
            String s1 = jsonString1.replaceAll("\"", "\\\"");
            System.out.println(s1);//输出json，封装数据完毕！

            //开始调用EAS接口操作：1、登录EAS（测试环境）  2、传参
            EASLoginProxyProxy login = new EASLoginProxyProxy();
            WSContext ctx = login.login("748", "riverdeepqwer12", "eas", "river001",
                    "L2", 2);
            System.out.println(ctx);
            if (null != ctx.getSessionId() && !"".equals(ctx.getSessionId())) {
                System.out.println("ctx.getSessionId():" + ctx.getSessionId());
                WSPortalMainFacadeSrvProxyProxy portal = new WSPortalMainFacadeSrvProxyProxy();
                String  remsg = portal.portalToEas("CW13",s1);
                System.out.println(remsg);//打印返回结果
            }
        }

        return McR.success();
    }

    //创建并设置整个JSON对象
    public static class MyJson{
        @JsonProperty("type")
        private String type;
        @JsonProperty("data")
        private List<ExpenseAccrualServiceImpl.Data> data;
        public String getType() {
            return type;
        }
        public void setType(String type) {
            this.type = type;
        }
        public List<ExpenseAccrualServiceImpl.Data> getData() {
            return data;
        }
        public void setData(List<ExpenseAccrualServiceImpl.Data> data) {
            this.data = data;
        }
    }

    //Data对象
    static class Data{
        @JsonProperty("head")
        private ExpenseAccrualServiceImpl.Head head;
        @JsonProperty("entry")
        private List<ExpenseAccrualServiceImpl.Entry> entry;
        /*get，set方法*/
        public ExpenseAccrualServiceImpl.Head getHead() {
            return head;
        }
        public void setHead(ExpenseAccrualServiceImpl.Head head) {
            this.head = head;
        }
        public List<ExpenseAccrualServiceImpl.Entry> getEntry() {
            return entry;
        }
        public void setEntry(List<ExpenseAccrualServiceImpl.Entry> entry) {
            this.entry = entry;
        }
    }

    //Head对象
    @AllArgsConstructor
    @NoArgsConstructor
    @lombok.Data
    static class Head{
        @JsonProperty("settlementType")
        private String settlementType;//付款方式

        @JsonProperty("number")
        private String number;//流程编号

        @JsonProperty("requestId")
        private String requestId;//Portal唯一性校验=实例ID

        @JsonProperty("appDate")
        private String appDate;//申请日期

        @JsonProperty("company")
        private String company;//所属公司

        @JsonProperty("appDepart")
        private String appDepart;//申请部门

        @JsonProperty("applicant")
        private String applicant;//申请人

        @JsonProperty("status")
        private String status;//审核状态=审核（默认）

    }

    //Entry对象
    @AllArgsConstructor
    @NoArgsConstructor
    @lombok.Data
    static class Entry {
        @JsonProperty("summary")
        private String summary;//财务摘要
        @JsonProperty("proNum")
        private String proNum;//项目编号
        @JsonProperty("amount")
        private String amount;//原币金额
        @JsonProperty("costCenter")
        private String costCenter;//成本中心
        @JsonProperty("campus")
        private String campus;//校区部门编码
        @JsonProperty("payCompany")
        private String payCompany;//付款主体
        @JsonProperty("title")
        private String title;//费用内容
        @JsonProperty("realAmount")
        private String realAmount;//折合人民币金额
        @JsonProperty("entryOwnId")
        private String entryOwnId;//分录唯一ID： 表单唯一识别码UUID
        @JsonProperty("exchangeRate")
        private String exchangeRate;//汇率
        @JsonProperty("currency")
        private String currency;//币别
        @JsonProperty("lines")
        private String lines;//业务线
        @JsonProperty("portalFeeType")
        private String portalFeeType;//费用编号
        @JsonProperty("supplierNo")
        private String supplierNo;//供应商编码
    }

    /**
     * 将时间戳转换为指定时区的格式化日期字符串。
     *
     * @param timestamp 时间戳（毫秒）
     * @param timeZone  时区ID，例如 "UTC" 或 "Asia/Shanghai"
     * @param pattern   日期格式，例如 "yyyy-MM-dd"
     * @return 格式化后的日期字符串
     */
    public  String convertTimestampToDateString(long timestamp, String timeZone, String pattern) {
        // 将时间戳转换为Instant
        Instant instant = Instant.ofEpochMilli(timestamp);
        // 将Instant转换为指定时区的日期
        LocalDate date = instant.atZone(ZoneId.of(timeZone)).toLocalDate();
        date = date.plusDays(1);
        // 创建一个DateTimeFormatter来格式化日期
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern(pattern);
        // 格式化LocalDate为字符串
        String formattedDate = date.format(formatter);
        // 返回格式化后的日期字符串
        return formattedDate;
    }



}
