package com.malk.ruisi.service.impl;

import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.malk.ruisi.client.WSContext;
import com.malk.ruisi.service.TravelWriteKingDeeService;
import com.malk.ruisi.services.EASLogin.EASLoginProxyProxy;
import com.malk.ruisi.services.WSPortalMainFacade.WSPortalMainFacadeSrvProxyProxy;
import com.malk.server.aliwork.YDConf;
import com.malk.server.aliwork.YDParam;
import com.malk.server.common.McR;
import com.malk.server.common.McREnum;
import com.malk.server.dingtalk.DDConf;
import com.malk.service.aliwork.YDClient;
import lombok.AllArgsConstructor;
import lombok.NoArgsConstructor;
import lombok.SneakyThrows;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.time.Instant;
import java.time.LocalDate;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

@Service
public class TravelWriteKingDeeServiceImpl implements TravelWriteKingDeeService {
    @Autowired
    private YDClient ydClient;
    @Autowired
    private YDConf ydConf;
    @Autowired
    private DDConf ddConf;

    @SneakyThrows
    @Override
    public McR insetXMC(String instanceId) throws JsonProcessingException {
        if(instanceId != null){
            System.out.println("实例ID："+instanceId);
            //通过实例ID拿取宜搭表单数据
            Map data = (Map) ydClient.queryData(YDParam.builder().formInstId(instanceId)
                    .appType(ydConf.getAppType()).systemToken(ydConf.getSystemToken())
                    .userId(ddConf.getOperator()).build(), YDConf.FORM_QUERY.retrieve_id).getFormData();
            System.out.println(data);
            if(data.get("radioField_luaxcrpq").toString().equals("差旅报销流程")){
                ObjectMapper objectMapper = new ObjectMapper();
                // 创建并设置head对象
                Head head = new Head();
                //根据是否境外公司判断head传参类型
                String requestId = null;
                if(data.get("selectField_luaxcrq1").toString().equals("是")){
                    head.setReservedField2(instanceId);//申请单实例

                    head.setTitle(data.get("textField_luaxcrpv").toString());//title

                    String s = data.get("serialNumberField_luaxcrpw").toString();
                    String prefix = "CW04-SLBX-";
                    String result = prefix+s;
                    head.setNumber(result);//number

                    String string = data.get("serialNumberField_luaxcrpw").toString();
                    requestId = string.substring(string.length() - 7);
                    head.setRequestId(requestId);//requestId  Portal做唯一性校验

                    head.setApplicant(data.get("textField_lwheukhv").toString());//申请人（编码）
                    head.setAppDepart(data.get("textField_lwheukhw").toString());//申请部门（编码）

                    long time = (long)data.get("dateField_luaxcrpz");
                    String utc = convertTimestampToDateString(time, "UTC", "yyyy-MM-dd");
                    head.setAppDate(utc);//申请日期

                    String company = data.get("textField_lwheukhx").toString();
                    head.setCompany(company);//所属公司（编码）

                    if(data.get("selectField_luaxcrq1").toString().equals('是')){ //是否境外公司（0-是  1-否）
                        head.setIsOffCompany("0");
                    }else{
                        head.setIsOffCompany("1");
                    }

                    if(data.get("selectField_luaxcrq2").toString().equals('是')){ //是否提供发票（0-是  1-否）
                        head.setIsInvoice("0");
                    }else{
                        head.setIsInvoice("1");
                    }

                    head.setCurrency(data.get("textField_lwhkeuno").toString());//币种
                    head.setExchangeRate(new BigDecimal(data.get("numberField_luaxcrq6").toString())
                            .setScale(6,BigDecimal.ROUND_HALF_UP).toString());//汇率:保留6位小数
                    head.setSettlementType(data.get("textField_lwhkeunp").toString());//付款方式（编码）
                    head.setAmount(new BigDecimal(data.get("numberField_luaxcrq8").toString())
                            .setScale(2,BigDecimal.ROUND_HALF_UP).toString());//折合人民币金额：保留2位小数
                    head.setRealAmount(new BigDecimal(data.get("numberField_luaxcrq9").toString())
                            .setScale(2,BigDecimal.ROUND_HALF_UP).toString());//本次付款金额：保留2位小数
                    head.setAcctName(data.get("textField_luaxcrqd").toString());//收款人户名
                    head.setAccount(data.get("textField_luaxcrqi").toString());//收款人账号

                    ObjectMapper mapp = new ObjectMapper();
                    String str = data.get("countrySelectField_lugqdy57").toString();
                    JsonNode rootNode = mapp.readTree(str);
                    String country = rootNode.get(0).asText();
                    head.setCountry(country);//收款方国家

                    head.setBank(data.get("textField_luaxcrqj").toString());//收款人开户银行bank
                    head.setReservedField1(data.get("selectField_lx02gcaw").toString());//用途---财务审批节点填写
                    head.setAddr(data.get("textField_luaxcrqm").toString());//地址
                    head.setSwift(data.get("textField_luaxcrqn").toString());//SWIFT
                    head.setInvoiceNum(data.get("textField_luaxcrqo").toString());//发票号码
                    head.setStatus("审核");
                }else {
                    head.setReservedField2(instanceId);//申请单实例

                    if(data.get("textField_luaxcrpv").toString() != null){
                        head.setTitle(data.get("textField_luaxcrpv").toString());//title
                    }

                    String s = data.get("serialNumberField_luaxcrpw").toString();
                    String prefix = "CW04-SLBX-";
                    String result = prefix+s;
                    head.setNumber(result);//number

                    String ss = data.get("serialNumberField_luaxcrpw").toString();
                    requestId = ss.substring(ss.length() - 7);
                    head.setRequestId(requestId);//requestId  Portal做唯一性校验

                    head.setApplicant(data.get("textField_lwheukhv").toString());//申请人（编码）
                    head.setAppDepart(data.get("textField_lwheukhw").toString());//申请部门（编码）

                    long time = (long)data.get("dateField_luaxcrpz");
                    String utc = convertTimestampToDateString(time, "UTC", "yyyy-MM-dd");
                    head.setAppDate(utc);//申请日期

                    String company = data.get("textField_lwheukhx").toString();
                    head.setCompany(company);//所属公司（编码）

                    if(data.get("selectField_luaxcrq1").toString().equals("是")){ //是否境外公司（0-是  1-否）
                        head.setIsOffCompany("0");
                    }else{
                        head.setIsOffCompany("1");
                    }

                    if(data.get("selectField_luaxcrq2").toString().equals("是")){ //是否提供发票（0-是  1-否）
                        head.setIsInvoice("0");
                    }else{
                        head.setIsInvoice("1");
                    }

                    head.setCurrency(data.get("textField_lwhkeuno").toString());//币种
                    head.setExchangeRate(new BigDecimal(data.get("numberField_luaxcrq6").toString())
                            .setScale(6,BigDecimal.ROUND_HALF_UP).toString());//汇率:保留6位小数
                    head.setSettlementType(data.get("textField_lwhkeunp").toString());//付款方式（编码）
                    head.setAmount(new BigDecimal(data.get("numberField_luaxcrq8").toString())
                            .setScale(2,BigDecimal.ROUND_HALF_UP).toString());//折合人民币金额：保留2位小数
                    head.setRealAmount(new BigDecimal(data.get("numberField_luaxcrq9").toString())
                            .setScale(2,BigDecimal.ROUND_HALF_UP).toString());//本次付款金额：保留2位小数
                    head.setAcctName(data.get("textField_luaxcrqd").toString());//收款人户名
                    head.setAccount(data.get("textField_luaxcrqi").toString());//收款人账号
                    head.setBank(data.get("textField_luaxcrqj").toString());//收款人开户银行
                    head.setReservedField1(data.get("selectField_lx02gcaw").toString());//用途
                    head.setBankNum(data.get("textField_lwhkeunq").toString());//收款人银行联行号

                    ObjectMapper mapper = new ObjectMapper();
                    String string = data.get("countrySelectField_lugqdy57").toString();
                    JsonNode rootNode = mapper.readTree(string);
                    String country = rootNode.get(0).asText();//收款方国家
                    head.setCountry(country);

                    String jsonString = data.get("addressField_luqo3eo1").toString();
                    ObjectMapper objectMapper1 = new ObjectMapper();
                    JsonNode jsonNode = objectMapper1.readTree(jsonString);
                    JsonNode regionTextArray  = jsonNode.get("regionText");
                    if (regionTextArray.isArray() && !regionTextArray.isEmpty()) {
                        JsonNode regionTextObject = regionTextArray.get(0);
                        String zhCN = regionTextObject.get("zh_CN").asText();
                        System.out.println(zhCN);
                        head.setProvince(zhCN);//收款方省市
                    }

                    String city = data.get("textField_lwgkgvcm").toString();
                    head.setCity(city);//收款方市县

                    head.setStatus("审核");
                }

                // 创建并设置entry对象列表
                ArrayList<Entry> entries = new ArrayList<>();
                Entry entry1 = new Entry();

                //获取子表单中字段信息
                List<Map> jsonString = (List<Map>)data.get("tableField_luap0c8x");
                System.out.println("jsonString:"+jsonString);
                int num=0;
                for (Map map1 : jsonString) {//遍历子表单数据
                    num++;
                    String entryOwnId = requestId.concat(String.valueOf(num));
                    entry1.setEntryOwnId(entryOwnId);//entryOwnId:分录唯一ID

                    String summary = data.get("textField_lx17qf5z").toString();
                    entry1.setSummary("支付-"+summary);//财务摘要

                    String lines = map1.get("textField_lwhkeunr").toString();//拿取到产品线【关联表单->文本】
                    entry1.setLines(lines);

                    String portalFeeType = map1.get("textField_lwhkeuns").toString();//费用编号【关联表单->文本】
                    entry1.setPortalFeeType(portalFeeType);

                    String proNum = map1.get("textField_lwhkeunt").toString();//拿取到项目编号【关联表单->文本】
                    entry1.setProNum(proNum);

                    String costCenter = map1.get("textField_lwhkeunu").toString();//成本中心
                    entry1.setCostCenter(costCenter);

                    if(map1.get("textField_lwhkeunv").toString() != null){
                        entry1.setCampus(map1.get("textField_lwhkeunv").toString());//校区部门
                    }

                    String payCompany = map1.get("textField_lwhkeunw").toString();//付款主体
                    entry1.setPayCompany(payCompany);

                    String payAmount = new BigDecimal(map1.get("numberField_luap0c94").toString())
                            .setScale(2,BigDecimal.ROUND_HALF_UP).toString();//付款金额
                    entry1.setPayAmount(payAmount);

                    String isSpeInvoice = map1.get("textField_lwhkeunx").toString();//是否增值税专用发票
                    entry1.setIsSpeInvoice(isSpeInvoice);

                    String taxAmount = new BigDecimal(map1.get("numberField_luap0c96").toString())
                            .setScale(2,BigDecimal.ROUND_HALF_UP).toString();//含税金额
                    entry1.setTaxAmount(taxAmount);

                    String rate = new BigDecimal(map1.get("numberField_lu94kny9").toString())
                            .setScale(2,BigDecimal.ROUND_HALF_UP).toString();//税率
                    entry1.setRate(rate);

                    String notTaxAmount = new BigDecimal(map1.get("numberField_lu94kny8").toString())
                            .setScale(2,BigDecimal.ROUND_HALF_UP).toString();//不含税金额
                    entry1.setNotTaxAmount(notTaxAmount);

                    String tax = new BigDecimal(map1.get("numberField_lu94knya").toString())
                            .setScale(2,BigDecimal.ROUND_HALF_UP).toString();//税额
                    entry1.setTax(tax);

                    entries.add(entry1);
                    num++;
                }

                // 创建并设置data对象
                Data data1 = new Data();
                data1.setHead(head);
                data1.setEntry(entries);
                // 创建并设置整个JSON对象
                MyJson json = new MyJson();
                json.setType("CW04");
                List<Data> dataList = new ArrayList<>();
                dataList.add(data1);
                json.setData(dataList);
                // 序列化对象到JSON字符串
                String jsonString1 = objectMapper.writeValueAsString(json);
                // 输出JSON字符串（参数封装完毕!）
                String s1 = jsonString1.replaceAll("\"", "\\\"");
                System.out.println(s1);
                //开始調用EAS系統接口
                EASLoginProxyProxy login = new EASLoginProxyProxy();
                WSContext ctx = login.login("748", "riverdeepqwer12", "eas", "river001",
                        "L2", 2);
                System.out.println(ctx);
                if (null != ctx.getSessionId() && !"".equals(ctx.getSessionId())) {
                    System.out.println("ctx.getSessionId():" + ctx.getSessionId());
                    WSPortalMainFacadeSrvProxyProxy portal = new WSPortalMainFacadeSrvProxyProxy();
                    try {
                        String remsg = portal.portalToEas("CW04", s1);
                        System.out.println("remsg:"+ remsg);
                    }catch (Exception e){
                        //异常处理
                        e.printStackTrace();
                    }
                }

            }

        }else {
            return McR.error(McREnum.UNKNOWN_EXCEPTION.toString(),"参数缺失");
        }
        return McR.success();
    }

    //创建并设置整个JSON对象
    public static class MyJson{
        @JsonProperty("type")
        private String type;
        @JsonProperty("data")
        private List<TravelWriteKingDeeServiceImpl.Data> data;
        public String getType() {
            return type;
        }
        public void setType(String type) {
            this.type = type;
        }
        public List<TravelWriteKingDeeServiceImpl.Data> getData() {
            return data;
        }
        public void setData(List<TravelWriteKingDeeServiceImpl.Data> data) {
            this.data = data;
        }
    }
    //Data对象
    static class Data{
        @JsonProperty("head")
        private TravelWriteKingDeeServiceImpl.Head head;
        @JsonProperty("entry")
        private List<TravelWriteKingDeeServiceImpl.Entry> entry;
        /*get，set方法*/
        public TravelWriteKingDeeServiceImpl.Head getHead() {
            return head;
        }
        public void setHead(TravelWriteKingDeeServiceImpl.Head head) {
            this.head = head;
        }
        public List<TravelWriteKingDeeServiceImpl.Entry> getEntry() {
            return entry;
        }
        public void setEntry(List<TravelWriteKingDeeServiceImpl.Entry> entry) {
            this.entry = entry;
        }
    }
    //Head对象
    @AllArgsConstructor
    @NoArgsConstructor
    @lombok.Data
    static class Head{
        @JsonProperty("reservedField2")
        private String reservedField2;//申请单实例
        @JsonProperty("title")
        private String title;//流程标题
        @JsonProperty("number")
        private String number;//流程编号
        @JsonProperty("requestId")
        private String requestId;//Portal唯一性校验=实例ID
        @JsonProperty("applicant")
        private String applicant;//申请人
        @JsonProperty("appDepart")
        private String appDepart;//申请部门
        @JsonProperty("appDate")
        private String appDate;//申请日期
        @JsonProperty("company")
        private String company;//所属公司
        @JsonProperty("isOffCompany")
        private String isOffCompany;//是否境外公司
        @JsonProperty("isInvoice")
        private String isInvoice;//是否提供发票
        @JsonProperty("currency")
        private String currency;//币种
        @JsonProperty("exchangeRate")
        private String exchangeRate;//汇率
        @JsonProperty("settlementType")
        private String settlementType;//付款方式
        @JsonProperty("amount")
        private String amount;//折合人民币金额
        @JsonProperty("realAmount")
        private String realAmount;//本次付款金额
        @JsonProperty("supplierNo")
        private String supplierNo;//供应商名称
        @JsonProperty("acctName")
        private String acctName;//收款人户名
        @JsonProperty("account")
        private String account;//收款人账号
        @JsonProperty("bank")
        private String bank;//收款人开户银行
        @JsonProperty("reservedField1")
        private String reservedField1;//用途
        @JsonProperty("bankNum")
        private String bankNum;//收款人银行联行号
        @JsonProperty("country")
        private String country;//收款方国家
        @JsonProperty("province")
        private String province;//收款方省
        @JsonProperty("city")
        private String city;//收款方市县
        @JsonProperty("addr")
        private String addr;//地址
        @JsonProperty("swift")
        private String swift;//SWIFT
        @JsonProperty("invoiceNum")
        private String invoiceNum;//invoiceNum
        @JsonProperty("status")
        private String status;//单据状态

    }
    //Entry对象
    @AllArgsConstructor
    @NoArgsConstructor
    @lombok.Data
    static class Entry {
        @JsonProperty("entryOwnId")
        private String entryOwnId;//表单唯一识别码UUID
        @JsonProperty("summary")
        private String summary;//财务摘要
        @JsonProperty("lines")
        private String lines;//产品线
        @JsonProperty("portalFeeType")
        private String portalFeeType;//费用编号
        @JsonProperty("proNum")
        private String proNum;//项目编号
        @JsonProperty("costCenter")
        private String costCenter;//成本中心
        @JsonProperty("campus")
        private String campus;//校区部门
        @JsonProperty("payCompany")
        private String payCompany;//付款主体
        @JsonProperty("payAmount")
        private String payAmount;//付款金额
        @JsonProperty("isSpeInvoice")
        private String isSpeInvoice;//是否增值税专用发票
        @JsonProperty("taxAmount")
        private String taxAmount;//含税金额
        @JsonProperty("rate")
        private String rate;//税率
        @JsonProperty("notTaxAmount")
        private String notTaxAmount;//不含税金额
        @JsonProperty("tax")
        private String tax;//税额
    }
    /**
     * 将时间戳转换为指定时区的格式化日期字符串。
     *
     * @param timestamp 时间戳（毫秒）
     * @param timeZone  时区ID，例如 "UTC" 或 "Asia/Shanghai"
     * @param pattern   日期格式，例如 "yyyy-MM-dd"
     * @return 格式化后的日期字符串
     */
    public  String convertTimestampToDateString(long timestamp, String timeZone, String pattern) {
        // 将时间戳转换为Instant
        Instant instant = Instant.ofEpochMilli(timestamp);
        // 将Instant转换为指定时区的日期
        LocalDate date = instant.atZone(ZoneId.of(timeZone)).toLocalDate();
        date = date.plusDays(1);
        // 创建一个DateTimeFormatter来格式化日期
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern(pattern);
        // 格式化LocalDate为字符串
        String formattedDate = date.format(formatter);
        // 返回格式化后的日期字符串
        return formattedDate;
    }


}
